#include <GDIPlus.au3>
#include <GUIConstantsEx.au3>

AutoItSetOption( "GUIOnEventMode", 1 )
AutoItSetOption( "MouseCoordMode", 2 )
AutoItSetOption( "MustDeclareVars", 1 )

Global $iW = 600, $iH = 400, $iBgColor = 0xF0F0F0
Global $hGUI = GUICreate("", $iW, $iH)

_GDIPlus_Startup()
Global $hGraphic = _GDIPlus_GraphicsCreateFromHWND($hGUI)
Global $hBlue = _GDIPlus_PenCreate(0xFF0000FF) ; Blue
Global $hRed = _GDIPlus_PenCreate(0xFFFF0000) ; Red
Global $hBitmap = _GDIPlus_BitmapCreateFromScan0($iW, $iH)
Global $hCtxt = _GDIPlus_ImageGetGraphicsContext($hBitmap)

Example()

Func Example()
	GUISetOnEvent($GUI_EVENT_CLOSE, "_Exit")
	GUISetBkColor($iBgColor, $hGUI)

	_GDIPlus_GraphicsSetSmoothingMode($hCtxt, 5)
	_GDIPlus_GraphicsSetPixelOffsetMode($hCtxt, 4)

	Local $iPosX = 25, $iPosY = 25, $iWidth = 550, $iHeight = 350
	Local $fDeg = ACos(-1) / 180, $fRad = 180 / ACos(-1), $iWh = $iW / 2, $iHh = $iH / 2
	Local $fDiameter = 30, $fRadius = $fDiameter / 2, _
	      $mpos, $mwx, $mwy, $pd, $fAngle, $fDistance, $x, $y

	GUISetState()
	Do
		; Square
		_GDIPlus_GraphicsClear($hCtxt, 0xFF000000 + $iBgColor)
		_GDIPlus_GraphicsDrawRect($hCtxt, $iPosX, $iPosY, $iWidth, $iHeight, $hBlue)

		; Blue circle
		_GDIPlus_GraphicsDrawEllipse($hCtxt , $iWh - $fRadius, $iHh - $fRadius, $fDiameter , $fDiameter , $hBlue)
		_GDIPlus_GraphicsDrawEllipse($hCtxt , $iWh  , $iHh  , 1 , 1 , $hBlue)

		; Red circle
		$mpos = MouseGetPos() ;get the mouse position
		$mwx = $mpos[0] - $iWh ;get the x coordinate
		$mwy = $iHh - $mpos[1] ;get the y coordinate
		$fAngle = 2 * -ATan($mwy / ($mwx + Sqrt($mwx * $mwx + $mwy * $mwy))) * $fRad ; calculate the angle between center and mouse position -> trigonometry ;-)

		$fDistance = $fRadius + $fRadius ;calculate the distance between both circles so that they always touch each other

		; Center of red circle
		If $fAngle == "-1.#IND" Then
			$x = 269
			$y = 200
		Else
			$x = $iWh + Cos($fAngle * $fDeg) * ($fDistance + 1)
			$y = $iHh + Sin($fAngle * $fDeg) * ($fDistance + 1)
		EndIf

		_GDIPlus_GraphicsDrawEllipse( $hCtxt, $x, $y, 1, 1, $hRed )                                         ; Draw center of red circle
		_GDIPlus_GraphicsDrawEllipse( $hCtxt, $x - $fRadius, $y - $fRadius, $fDiameter, $fDiameter, $hRed ) ; Draw red circle
		_GDIPlus_GraphicsDrawline( $hCtxt, $x, $y, 300, 200, $hRed )                                        ; Draw line from red center to blue center
		_GDIPlus_GraphicsDrawImageRect( $hGraphic, $hBitmap, 0, 0, $iW, $iH )                               ; Draw square
	Until Not Sleep(10)
EndFunc

Func _Exit()
	_GDIPlus_GraphicsDispose($hGraphic)
	_GDIPlus_GraphicsDispose($hCtxt)
	_GDIPlus_BitmapDispose($hBitmap)
	_GDIPlus_PenDispose($hBlue)
	_GDIPlus_PenDispose($hRed)
	_GDIPlus_Shutdown()
	GUIDelete()
	Exit
EndFunc
